"""placeholder

Revision ID: 62bcf2610c5d
Revises: 7b696fd94949
Create Date: 2019-12-03 00:56:52.488257

"""

# revision identifiers, used by Alembic.
revision = '62bcf2610c5d'
down_revision = '7b696fd94949'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
