"""placeholder

Revision ID: 7a4fd0fc3f8c
Revises: 62bcf2610c5d
Create Date: 2019-12-03 00:57:01.579012

"""

# revision identifiers, used by Alembic.
revision = '7a4fd0fc3f8c'
down_revision = '62bcf2610c5d'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
