"""placeholder

Revision ID: 7b696fd94949
Revises: 22fb1af2d51e
Create Date: 2019-12-03 00:56:34.006320

"""

# revision identifiers, used by Alembic.
revision = '7b696fd94949'
down_revision = '22fb1af2d51e'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
